from contextlib import AsyncExitStack

from v6d3music.config import myroot
from v6d3music.utils.presets import allowed_effects

from ptvp35 import *
from v6d2ctx.context import Explicit

__all__ = ('DefaultEffects',)


class DefaultEffects:
    def get(self, gid: int) -> str | None:
        effects = self.__db.get(gid, None)
        if effects in allowed_effects:
            return effects
        else:
            return None

    async def set(self, gid: int, effects: str | None) -> None:
        if effects is not None and effects not in allowed_effects:
            raise Explicit('these effects are not allowed')
        await self.__db.set(gid, effects)

    async def __aenter__(self) -> 'DefaultEffects':
        async with AsyncExitStack() as es:
            self.__db = await es.enter_async_context(DbFactory(myroot / 'effects.db', kvfactory=KVJson()))
            self.__es = es.pop_all()
            return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        async with self.__es:
            del self.__es
